//------------------------------------------------------------------------------
//
// Copyright (c) 2004-2005  Matt Brunk  <info@smartcache.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License VERSION 2 as
// published by the Free Software Foundation.  You are not allowed to
// use any other version of the license; unless you got the explicit
// permission from the author to do so.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//------------------------------------------------------------------------------
#include "stdafx.h"
#include <stdlib.h>
#include "ACOS1.h"
#include "GUID.h"

enum
{
	FILE_MANUFACTURER		= 0xFF01,
	FILE_PERSONALIZATION	= 0xFF02,
	FILE_SECURITY			= 0xFF03,
	FILE_USER_MANAGEMENT	= 0xFF04
};

static U8 DEFAULT_ATR[] = 
{0x3B, 0xBE, 0x11, 0x00, 0x00, 0x41, 0x01, 0x38, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x90, 0x00};

//static U8 DEFAULT_IC[8] = {'A', 'C', 'O', 'S', 'T', 'E', 'S', 'T'}; 
static U8 DEFAULT_IC[8] = {'A', 'C', 'O', 'S', 'T', 'E', 'S', 'T'}; 


static UINT MAX_INCORRECT_ICS = 4;

CACOS1::CACOS1()
{
	NewCard();
}

CACOS1::~CACOS1()
{

}

void CACOS1::SetATR(U8 *atr, UINT length)
{
	ASSERT(length <= MAX_ATR_LENGTH);

	::CopyMemory(m_atr, atr, length);
	m_iAtrLength = length;

}

void CACOS1::GetATR(U8 *atr, UINT &length)
{
	::CopyMemory(atr, m_atr, m_iAtrLength);
	length = m_iAtrLength;

}

void CACOS1::GetDefaultATR(U8 *atr, UINT &length)
{
	::CopyMemory(atr, DEFAULT_ATR, sizeof DEFAULT_ATR);
	length = sizeof DEFAULT_ATR;

}

void CACOS1::NewCard()
{
	m_eStage = STAGE_MANUFACTURING;

	::CopyMemory(m_atr, DEFAULT_ATR, sizeof DEFAULT_ATR);
	m_iAtrLength = sizeof DEFAULT_ATR;
	::CopyMemory(m_password, DEFAULT_IC, sizeof DEFAULT_IC);
	m_passwordLength = sizeof DEFAULT_IC;

	m_eStage = m_atr[ATR_LIFECYCLE_STAGE_IDX];

	::ZeroMemory(m_manufacturingFile, sizeof m_manufacturingFile);
	m_manufacturingFile[0][0] = 0x20;
	
	::ZeroMemory(m_personalizationFile, sizeof m_personalizationFile);
	::ZeroMemory(m_securityFile, sizeof m_securityFile);

	m_iNumIncorrectICs	= 0;
	m_bFileSelected		= false;
	m_N_OF_FILE			= 0;

	for ( size_t nn = 0; nn < m_userFiles.size(); nn++ )
	{
		delete [] m_userFiles[nn].data;
	}

	unsigned char guid[GUID_LENGTH];
	CGUID::MakeGUID(guid, true);
	::CopyMemory(m_RNDc, guid, sizeof m_RNDc);

	m_userFiles.clear();
}

void CACOS1::ProcessCommand(U8 *cmd, U8 *response, UINT &responseLength)
{
	::CopyMemory(m_cmd, cmd, 5);

	//TRACE("cmd = %02X\n", cmd[1]);

	if ( m_cmd[0] != 0x80 )
	{
		responseLength = 0;
	}
	else
	{
		response[0]		= cmd[1];
		responseLength	= 1;

		switch ( m_cmd[1] )
		{
			case INS_READ_RECORD:
			{

				UINT recNum = m_cmd[2];
				UINT length = m_cmd[4];

				if ( m_bFileSelected )
				{
					switch ( m_currFile )
					{
						case FILE_MANUFACTURER:
						{
							if ( recNum == 0 || recNum == 1 )
							{
								if ( length >= 1 && length <= 8 )
								{
									::CopyMemory(&response[1], m_manufacturingFile[recNum], length);
									response[1 + length]		= 0x90;
									response[1 + length + 1]	= 0x00;
									responseLength = 1 + length + 2;
								}
								else
								{
									response[1] = 0x67;
									response[2] = 0x00;
									responseLength = 3;
								}
							}
							else
							{
								response[1] = 0x6A;
								response[2] = 0x83;
								responseLength = 3;
							}
						}
						break;

						case FILE_PERSONALIZATION:
						{
							if ( recNum < 3 )
							{
								if ( length >= 1 && length <= 4 )
								{
									::CopyMemory(&response[1], m_personalizationFile[recNum], length);
									response[1 + length]		= 0x90;
									response[1 + length + 1]	= 0x00;
									responseLength = 1 + length + 2;
								}
								else
								{
									response[1] = 0x67;
									response[2] = 0x00;
									responseLength = 3;
								}
							}
							else
							{
								response[1] = 0x6A;
								response[2] = 0x83;
								responseLength = 3;
							}
						}
						break;

						case FILE_USER_MANAGEMENT:
						{
							if ( recNum < m_userFiles.size() )
							{
								::CopyMemory(&response[1], &m_userFiles[recNum].ufd, length);
								response[1 + length]		= 0x90;
								response[1 + length + 1]	= 0x00;
								responseLength = 1 + length + 2;

							}
							else
							{
								response[1] = 0x6A;
								response[2] = 0x83;
								responseLength = 3;
							}
						}
						break;

						case FILE_SECURITY:
						{
							if ( recNum < 14 )
							{
								if ( length >= 1 && length <= 8 )
								{

									::CopyMemory(&response[1], m_securityFile[recNum], length);
									response[1 + length]		= 0x90;
									response[1 + length + 1]	= 0x00;
									responseLength = 1 + length + 2;
								}
								else
								{
									response[1] = 0x67;
									response[2] = 0x00;
									responseLength = 3;
								}
							}
							else
							{
								response[1] = 0x6A;
								response[2] = 0x83;
								responseLength = 3;
							}
						}
						break;

						default:
						{


							bool found = false;
							size_t idx = 0;
							U16 file = ((m_currFile & 0xFF) << 8) | (m_currFile >> 8);
							for ( size_t nn = 0; nn < m_userFiles.size(); nn++ )
							{
								if ( m_userFiles[nn].ufd.fileIdentifier == file )
								{
									found = true;
									idx = nn;
									break;
								}
							}

							if ( found )
							{
								UserFileDefinitionStruct ufd = m_userFiles[idx].ufd;

								if ( recNum < ufd.numRecords )
								{
									if ( length <= ufd.recordLength )
									{
										//TRACE("read file %04X recNum %d bytes %d\n",
										//	m_currFile, recNum, length);

										UINT offset = recNum * ufd.recordLength;

										::CopyMemory(&response[1], &m_userFiles[idx].data[offset], length);
										response[1 + length]		= 0x90;
										response[1 + length + 1]	= 0x00;
										responseLength = 1 + length + 2;
									}
									else
									{
										response[1] = 0x67;
										response[2] = 0x00;
										responseLength = 3;
									}
								}
								else
								{
									response[1] = 0x6A;
									response[2] = 0x83;
									responseLength = 3;

								}
							}
							else
							{
								response[1] = 0x69;
								response[2] = 0x85;
								responseLength = 3;
							}
						}
						break;
					}
				}
				else
				{
					response[1] = 0x69;
					response[2] = 0x85;
					responseLength = 3;
				}
			}
			break;

			case INS_START_SESSION:
			{
				// Generate random key.
					
				::CopyMemory(&response[1], m_RNDc, 8);
				response[1 + 8]		= 0x90;
				response[1 + 8 + 1]	= 0x00;
				responseLength = 1 + 8 + 2;

			}
			break;

			case INS_GET_RESPONSE:
			{
				// Generate random key.
		
				
				// Left half of KS = 3-DES (3-DES (RNDC, #KC), #KT) & Right half of KS = 3-DES (RND T, #(reverseKT)),
				// where, #(reverse KT) is obtained by exchanging the Left and Right half of KT
				CTripleDES	desKc(m_securityFile[2], m_securityFile[12]);
				CTripleDES	desKt(m_securityFile[3], m_securityFile[13]);
				
				TripleDesBufType	desRNDcKc;

				desKc.Encrypt(m_RNDc, desRNDcKc);
				desKt.Encrypt(desRNDcKc, m_Ks_left);

				CTripleDES	desKtReverse(m_securityFile[13], m_securityFile[3]);
				desKtReverse.Encrypt(m_RNDt, m_Ks_right);

				// Verify DES (RNDT, #KS)
				TripleDesBufType	temp;
				CTripleDES	desKs(m_Ks_left, m_Ks_right);
				desKs.Encrypt(m_RNDt, temp);


				::CopyMemory(&response[1], temp, 8);
				response[1 + 8]		= 0x90;
				response[1 + 8 + 1]	= 0x00;
				responseLength = 1 + 8 + 2;

			}
			break;
		
			default:
			{

			}
			break;

		}
	}
}

void CACOS1::ProcessCommandData(U8 *data, UINT length, U8 *response, UINT &responseLength)
{
	responseLength = 0;

	switch ( m_cmd[1] )
	{
		case INS_SUBMIT_CODE:
		{
			bool pass = false;
			if ( m_iNumIncorrectICs >= MAX_INCORRECT_ICS )
			{
				response[0] = 0x69;
				response[1] = 0x83;
				responseLength = 2;
			}
			else if ( (m_eStage == STAGE_USER) && (m_personalizationFile[0][0] & 0x02)  )
			{
				CTripleDES	desKs(m_Ks_left, m_Ks_right);

				TripleDesBufType	desIcKs;

				desKs.Encrypt(m_securityFile[0], desIcKs);

				if ( memcmp(desIcKs, data, 8) == 0 )
				{
					pass = true;
				}
			}
			else
			{
				if ( (m_passwordLength == length) && (memcmp(m_password, data, m_passwordLength) == 0) )
				{
					pass = true;
				}
			}

			if ( pass )
			{
				response[0] = 0x90;
				response[1] = 0x00;
				responseLength = 2;

			}
			else
			{
				m_iNumIncorrectICs++;
				if ( m_iNumIncorrectICs >= MAX_INCORRECT_ICS )
				{
					response[0] = 0x69;
					response[1] = 0x83;
					responseLength = 2;
				}
				else
				{
					response[0] = 0x63;
					response[1] = (U8)(0xC0 + (MAX_INCORRECT_ICS - m_iNumIncorrectICs));
					responseLength = 2;
				}
			}
		}
		break;
	

		case INS_SELECT_FILE:
		{
			// Validate file.

			U8 *p = (U8*)&m_currFile;

			p[0] = data[1];
			p[1] = data[0];

			if ( IsValidFile(m_currFile) )
			{
				response[0] = 0x90;
				response[1] = 0x00;
				responseLength = 2;
				m_bFileSelected = true;
			}
			else
			{
				response[0] = 0x6A;
				response[1] = 0x82;
				responseLength = 2;
			}

		}
		break;
		
		case INS_READ_RECORD:
		case INS_START_SESSION:
		case INS_GET_RESPONSE:
		{

		}
		break;

		case INS_WRITE_RECORD:
		{
			response[0]		= m_cmd[1];
			responseLength	= 1;

			UINT recNum = m_cmd[2];

			if ( m_bFileSelected )
			{
				switch ( m_currFile )
				{
					case FILE_MANUFACTURER:
					{
						if ( recNum == 0 || recNum == 1 )
						{
							if ( length >= 1 && length <= 8 )
							{
								::CopyMemory(m_manufacturingFile[recNum], data, length);
								response[0]	= 0x90;
								response[1]	= 0x00;
								responseLength = 2;

								if ( (recNum == 0) && (data[0] & 0x80) )
								{
									m_atr[ATR_LIFECYCLE_STAGE_IDX] = STAGE_PERSONALIZATION;
								}
							}
							else
							{
								response[0] = 0x67;
								response[1] = 0x00;
								responseLength = 2;
							}
						}
						else
						{
							response[0] = 0x6A;
							response[1] = 0x83;
							responseLength = 2;
						}
					}
					break;

					case FILE_PERSONALIZATION:
					{
						if ( recNum < 3 )
						{
							if ( length >= 1 && length <= 4 )
							{

								if ( (recNum == 0) && (length >= 3) )
								{
									UserFileStruct uf = {{0}, NULL};

									if ( m_N_OF_FILE != data[2] )
									{
										m_N_OF_FILE = data[2];
										m_userFiles.clear();
										//TRACE("init m_userFiles\n");
										for ( size_t nn = 0; nn < m_N_OF_FILE; nn++ )
										{
											m_userFiles.push_back(uf);
										}
									}
								}

								::CopyMemory(m_personalizationFile[recNum], data, length);
								response[0]	= 0x90;
								response[1]	= 0x00;
								responseLength = 2;

								if ( (recNum == 0) && (length == 4) && (data[3] & 0x80) )
								{
									m_atr[ATR_LIFECYCLE_STAGE_IDX] = STAGE_USER;
								}

								::CopyMemory(&m_atr[8], m_personalizationFile[0], 3);
								::CopyMemory(&m_atr[11], m_personalizationFile[1], 5);
							}
							else
							{
								response[0] = 0x67;
								response[1] = 0x00;
								responseLength = 2;
							}
						}
						else
						{
							response[0] = 0x6A;
							response[1] = 0x83;
							responseLength = 2;
						}
					}
					break;

					case FILE_USER_MANAGEMENT:
					{
						if ( recNum < m_N_OF_FILE )
						{
							if ( length >= 1 && length <= 6 && (m_N_OF_FILE <= m_userFiles.size()) )
							{
								UserFileDefinitionStruct ufd = m_userFiles[recNum].ufd;

								::CopyMemory(&ufd, data, length);
								m_userFiles[recNum].ufd = ufd;

								//TRACE("alloc data for file %d\n", recNum);
								if ( length == 6 )
								{
									m_userFiles[recNum].data = new U8[ufd.numRecords * ufd.recordLength];
								}

								response[0]	= 0x90;
								response[1]	= 0x00;
								responseLength = 2;
							}
							else
							{
								response[0] = 0x67;
								response[1] = 0x00;
								responseLength = 2;
							}
						}
						else
						{
							response[0] = 0x6A;
							response[1] = 0x83;
							responseLength = 2;
						}
					}
					break;

					case FILE_SECURITY:
					{
						if ( recNum < 14 )
						{
							if ( (length >= 1) && (length <= 8) )
							{

								::CopyMemory(m_securityFile[recNum], data, length);

								if ( recNum == 0 )
								{
									::CopyMemory(m_password, data, length);
									m_passwordLength = length;
								}
								response[0]	= 0x90;
								response[1]	= 0x00;
								responseLength = 2;
							}
							else
							{
								response[0] = 0x67;
								response[1] = 0x00;
								responseLength = 2;
							}
						}
						else
						{
							response[0] = 0x6A;
							response[1] = 0x83;
							responseLength = 2;
						}
					}
					break;

					default:
					{
						bool found = false;
						size_t idx = 0;
						U16 file = ((m_currFile & 0xFF) << 8) | (m_currFile >> 8);
						for ( size_t nn = 0; nn < m_userFiles.size(); nn++ )
						{
							if ( m_userFiles[nn].ufd.fileIdentifier == file )
							{
								found = true;
								idx = nn;
								break;
							}
						}

						if ( found )
						{
							UserFileDefinitionStruct ufd = m_userFiles[idx].ufd;

							if ( recNum < ufd.numRecords )
							{
								if ( length <= ufd.recordLength )
								{
									//TRACE("write file %04X recNum %d bytes %d\n",
									//	m_currFile, recNum, length);

									UINT offset = recNum * ufd.recordLength;

									::CopyMemory(&m_userFiles[idx].data[offset], data, length);
									response[0] = 0x90;
									response[1] = 0x00;
									responseLength = 2;
								}
								else
								{
									response[0] = 0x67;
									response[1] = 0x00;
									responseLength = 2;
								}
							}
							else
							{
								response[0] = 0x6A;
								response[1] = 0x83;
								responseLength = 2;

							}
						}
						else
						{
							response[0] = 0x69;
							response[1] = 0x85;
							responseLength = 2;
						}
					}
					break;
				}
			}
			else
			{
				response[1] = 0x69;
				response[2] = 0x85;
				responseLength = 3;
			}
		}
		break;


		case INS_AUTHENTICATE:
		{
			TripleDesBufType	desRNDcKt;	
			CTripleDES			desKt(m_securityFile[3], m_securityFile[13]);

			desKt.Encrypt(m_RNDc, desRNDcKt);


			if ( (memcmp(desRNDcKt, data, 8) == 0) )
			{
				response[0] = 0x61;
				response[1] = 0x08;
				responseLength = 2;

				::CopyMemory(m_RNDt, &data[8], 8);
			}
			else
			{
				m_iNumIncorrectICs++;
				if ( m_iNumIncorrectICs >= MAX_INCORRECT_ICS )
				{
					response[0] = 0x69;
					response[1] = 0x83;
					responseLength = 2;
				}
				else
				{
					response[0] = 0x63;
					response[1] = (U8)(0xC0 + (MAX_INCORRECT_ICS - m_iNumIncorrectICs));
					responseLength = 2;
				}
			}
		}
		break;
	


		default:
		{
			response[0] = 0x90;
			response[1] = 0x00;
			responseLength = 2;
		}
		break;
	}

}

bool CACOS1::IsValidFile(U16 file)
{
	bool valid = false;

	switch ( file )
	{
		case FILE_MANUFACTURER:
		case FILE_PERSONALIZATION:
		case FILE_SECURITY:
		case FILE_USER_MANAGEMENT:
		{
			valid = true;
		}
		break;

		default:
		{
			file = ((file & 0xFF) << 8) | (file >> 8);
			for ( size_t nn = 0; nn < m_userFiles.size(); nn++ )
			{
				if ( m_userFiles[nn].ufd.fileIdentifier == file )
				{
					valid = true;
					break;
				}
			}
		}
		break;
	}

	return valid;
}

void CACOS1::DoATR()
{
	m_bFileSelected = false;
	m_eStage = m_atr[ATR_LIFECYCLE_STAGE_IDX];
	m_N_OF_FILE = m_personalizationFile[0][2];
}
