//------------------------------------------------------------------------------
//
// Copyright (c) 2004-2005  Matt Brunk  <info@smartcache.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License VERSION 2 as
// published by the Free Software Foundation.  You are not allowed to
// use any other version of the license; unless you got the explicit
// permission from the author to do so.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//------------------------------------------------------------------------------
// DlgATR.cpp : implementation file
//

#include "stdafx.h"
#include "softcard.h"
#include "DlgATR.h"
#include "SoftCardTypes.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDlgATR dialog


CDlgATR::CDlgATR(CACOS1 *pAcos1, CWnd* pParent /*=NULL*/)
	: CDialog(CDlgATR::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDlgATR)
	//}}AFX_DATA_INIT

	m_pAcos1 = pAcos1;
	::ZeroMemory(&m_atr, sizeof m_atr);
}


void CDlgATR::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDlgATR)
	DDX_Control(pDX, IDC_EDIT_N_OF_FILE, m_editN_OF_FILE);
	DDX_Control(pDX, IDC_RADIO_USER, m_radioUser);
	DDX_Control(pDX, IDC_EDIT_OTHER_STAGE, m_editOtherStage);
	DDX_Control(pDX, IDC_RADIO_OTHER_STAGE, m_radioOtherStage);
	DDX_Control(pDX, IDC_RADIO_PERSONALIZATION, m_radioPersonalization);
	DDX_Control(pDX, IDC_RADIO_OTHER, m_radioOther);
	DDX_Control(pDX, IDC_RADIO_MANUFACTURING, m_radioManufacturing);
	DDX_Control(pDX, IDC_RADIO_ACOS1, m_radioAcos1);
	DDX_Control(pDX, IDC_EDIT_REVISION, m_editRevision);
	DDX_Control(pDX, IDC_EDIT_OTHER_ATR, m_editOtherATR);
	DDX_Control(pDX, IDC_EDIT_ACOS1_ATR, m_editAcos1ATR);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDlgATR, CDialog)
	//{{AFX_MSG_MAP(CDlgATR)
	ON_BN_CLICKED(IDC_BUTTON_RESET_ACOS1, OnButtonResetAcos1)
	ON_BN_CLICKED(IDC_RADIO_ACOS1, OnRadioAcos1)
	ON_BN_CLICKED(IDC_RADIO_OTHER, OnRadioOther)
	ON_BN_CLICKED(IDC_RADIO_MANUFACTURING, OnRadioStage)
	ON_BN_CLICKED(IDC_RADIO_PERSONALIZATION, OnRadioStage)
	ON_BN_CLICKED(IDC_RADIO_USER, OnRadioStage)
	ON_BN_CLICKED(IDC_RADIO_OTHER_STAGE, OnRadioStage)
	//}}AFX_MSG_MAP
	ON_WM_HEX_EDIT_UPDATE(OnHexEditUpdate)	//lint !e1773: Attempt to cast away const (or volatile)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDlgATR message handlers

BOOL CDlgATR::OnInitDialog() 
{
	CDialog::OnInitDialog();

	CClientDC dc(this);
	int	nHeight = -((dc.GetDeviceCaps(LOGPIXELSY) * 10) / 72);

	m_font.CreateFont
	(
		nHeight,
		0,
		0,
		0,
		FW_NORMAL,
		0,
		0,
		0,
		DEFAULT_CHARSET,
		OUT_CHARACTER_PRECIS,
		CLIP_CHARACTER_PRECIS,
		DEFAULT_QUALITY,
		DEFAULT_PITCH | FF_DONTCARE,
		_T("Courier New")
	);
	

	m_editAcos1ATR.SetFont(&m_font);
	m_editOtherATR.SetFont(&m_font);
	m_editRevision.SetFont(&m_font);

	m_editAcos1ATR.SetAllowSpaces(true);
	m_editOtherATR.SetAllowSpaces(true);

	m_editOtherStage.SetLimitText(2);
	m_editRevision.SetLimitText(2);
	m_editN_OF_FILE.SetLimitText(2);

	m_radioAcos1.SetCheck(m_atr.bAcos1);
	m_radioOther.SetCheck(!m_atr.bAcos1);

	UpdateAcos1();

	m_editRevision.SetOwner(m_hWnd);
	m_editOtherStage.SetOwner(m_hWnd);
	m_editN_OF_FILE.SetOwner(m_hWnd);

	return TRUE;
}

void CDlgATR::UpdateAcos1()
{
	CString	sss;

	m_atr.revision = m_atr.acos1Atr[ATR_REVISION_IDX];
	sss.Format(_T("%02X"), m_atr.revision);
	m_editRevision.SetWindowText(sss);

	sss.Format(_T("%02X"), m_atr.acos1Atr[ATR_N_OF_FILE_IDX]);
	m_editN_OF_FILE.SetWindowText(sss);

	m_atr.stage = m_atr.acos1Atr[ATR_LIFECYCLE_STAGE_IDX];

	m_radioManufacturing.SetCheck(m_atr.stage == STAGE_MANUFACTURING);
	m_radioPersonalization.SetCheck(m_atr.stage == STAGE_PERSONALIZATION);
	m_radioUser.SetCheck(m_atr.stage == STAGE_USER);
	m_radioOtherStage.SetCheck(m_atr.stage > STAGE_PERSONALIZATION);

	SetAtr(m_editAcos1ATR, m_atr.acos1Atr, m_atr.iAcos1AtrLength);
	SetAtr(m_editOtherATR, m_atr.otherAtr, m_atr.iOtherAtrLength);
}

void CDlgATR::SetAtr(CHexEdit &edit, U8 *buf, UINT length)
{
	CString sss;
	CString ttt;
	for ( UINT ii = 0; ii < length; ii++ )
	{
		ttt.Format(_T("%02X"), buf[ii]);
		if ( ii < (length - 1) )
		{
			ttt += _T(" ");
		}
		sss += ttt;
	}

	edit.SetWindowText(sss);
}

void CDlgATR::OnButtonResetAcos1() 
{
	m_pAcos1->GetDefaultATR(m_atr.acos1Atr, m_atr.iAcos1AtrLength);
	UpdateAcos1();
}

void CDlgATR::OnRadioAcos1() 
{
	// TODO: Add your control notification handler code here
	
}

void CDlgATR::OnRadioOther() 
{
	// TODO: Add your control notification handler code here
	
}

void CDlgATR::OnRadioStage() 
{
	if ( m_radioManufacturing.GetCheck() )
	{
		m_atr.stage = STAGE_MANUFACTURING;
	}
	else if ( m_radioPersonalization.GetCheck() )
	{
		m_atr.stage = STAGE_PERSONALIZATION;
	}
	else if ( m_radioUser.GetCheck() )
	{
		m_atr.stage = STAGE_USER;
	}
	else
	{
		CString sss;
		int		ii = m_atr.acos1Atr[ATR_LIFECYCLE_STAGE_IDX];
		m_editOtherStage.GetWindowText(sss);
		if ( !sss.IsEmpty() )
		{
			_stscanf(sss, "%x", &ii);
		}
		else
		{
			ii = 0;
		}
		m_atr.stage = ii;
	}

	m_atr.acos1Atr[ATR_LIFECYCLE_STAGE_IDX] = (U8)m_atr.stage;
	SetAtr(m_editAcos1ATR, m_atr.acos1Atr, m_atr.iAcos1AtrLength);
	
}

LRESULT CDlgATR::OnHexEditUpdate(WPARAM /*wParam*/, LPARAM lParam)
{
	if ( lParam == IDC_EDIT_OTHER_STAGE )
	{
		OnRadioStage();
	}
	else if ( lParam == IDC_EDIT_REVISION )
	{
		CString sss;
		int		ii = m_atr.acos1Atr[ATR_REVISION_IDX];
		m_editRevision.GetWindowText(sss);
		if ( !sss.IsEmpty() )
		{
			_stscanf(sss, "%x", &ii);
		}
		else
		{
			ii = 0;
		}
		m_atr.revision = (U8)ii;
		m_atr.acos1Atr[ATR_REVISION_IDX] = m_atr.revision;
		SetAtr(m_editAcos1ATR, m_atr.acos1Atr, m_atr.iAcos1AtrLength);
	}
	else if ( lParam == IDC_EDIT_N_OF_FILE )
	{
		CString sss;
		int		ii = m_atr.acos1Atr[ATR_N_OF_FILE_IDX];
		m_editN_OF_FILE.GetWindowText(sss);
		if ( !sss.IsEmpty() )
		{
			_stscanf(sss, "%x", &ii);
		}
		else
		{
			ii = 0;
		}
		m_atr.acos1Atr[ATR_N_OF_FILE_IDX] = (U8)ii;
		SetAtr(m_editAcos1ATR, m_atr.acos1Atr, m_atr.iAcos1AtrLength);
	}

	return 0;
}
