//------------------------------------------------------------------------------
//
// Copyright (c) 2004-2005  Matt Brunk  <info@smartcache.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License VERSION 2 as
// published by the Free Software Foundation.  You are not allowed to
// use any other version of the license; unless you got the explicit
// permission from the author to do so.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//------------------------------------------------------------------------------
#include "stdafx.h"
#include "MyTime.h"

#define NSECS_PER_SEC_REAL		1000000000.0

static BOOL				G_is_first_time = TRUE;	
static BOOL				G_did_query = FALSE;	
static BOOL				G_use_performance_counter = FALSE;	
static LARGE_INTEGER	G_freq;	

double MyTime::GetTime()
{
	static LARGE_INTEGER	initial_time;
	LARGE_INTEGER			liCurrent;

	if ( !G_did_query )
	{
		G_freq.QuadPart = 1;
		if ( ::QueryPerformanceFrequency(&G_freq) )
		{
			G_use_performance_counter = TRUE;
		}

		G_did_query = TRUE;
	}


	if ( G_use_performance_counter )
	{
		QueryPerformanceCounter(&liCurrent);
		
		if ( G_is_first_time )
		{
			G_is_first_time    = FALSE;
			initial_time       = liCurrent;
			liCurrent.QuadPart = 0;
		}
		else
		{
			liCurrent.QuadPart -= initial_time.QuadPart;
		}

		//--------------------------------------------------------------------------
		//	Change the current time into a number of seconds with a fraction.  
		//	Remember that the FileTime is in hundreds of nanoseconds.
		//--------------------------------------------------------------------------
		return (double)liCurrent.QuadPart / G_freq.QuadPart;
	}
	else
	{


		SYSTEMTIME		stCurrent;
		FILETIME		ftCurrent;

		//--------------------------------------------------------------------------
		//	Get the current time and convert to filetime
		//--------------------------------------------------------------------------
		GetLocalTime( &stCurrent ); 
		SystemTimeToFileTime( &stCurrent, &ftCurrent );
		
		//--------------------------------------------------------------------------
		//	Move the filetime into a large integer
		//--------------------------------------------------------------------------
		liCurrent.LowPart  = ftCurrent.dwLowDateTime;
		liCurrent.HighPart = ftCurrent.dwHighDateTime;

		if ( G_is_first_time )
		{
			G_is_first_time    = FALSE;
			initial_time       = liCurrent;
			liCurrent.QuadPart = 0;
		}
		else
		{
			liCurrent.QuadPart -= initial_time.QuadPart;
		}

		//--------------------------------------------------------------------------
		//	Change the current time into a number of seconds with a fraction.  
		//	Remember that the FileTime is in hundreds of nanoseconds.
		//--------------------------------------------------------------------------
		return (double)liCurrent.QuadPart / ( NSECS_PER_SEC_REAL / 100.0 );
	}
}

void MyTime::Reset()
{
	G_is_first_time = TRUE;
}


void MyTime::Delay(unsigned long milliseconds)
{
	if ( milliseconds == 0 )
	{
		return;
	}

	double startTime = GetTime();
	while ( (GetTime() - startTime) < (milliseconds / 1000.0) )
	{
		;
	}
}

